/*
 * Copyright (c) 2018, Larry Lile <lile@users.sourceforge.net>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice unmodified, this list of conditions, and the following
 *    disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id$
 */

#include "config.h"

#define PASSWORD_BUFSIZE 1024

ssize_t my_getpass(char *prompt, char **lineptr, size_t * n, FILE * stream);
int debug_msg(int level, const char *fmt, ...);

typedef struct {
	char *user;
	char *userdn;
	BerValue cred;
} LDAPAuth;

typedef struct {
	char *url;
	char *base;
	int scope;
	char *tls_cacertfile;
	char *tls_cacertdir;
	char *tls_reqcert;
	LDAPAuth auth;
	int version;
} LDAPConfig;

typedef struct {
	char *dn;
	char *uid;
	char *gecos;
	char *loginShell;
} USEREntry;

int get_ldap_config(LDAPConfig * config);
int rebindproc(LDAP * ld, LDAP_CONST char *url, ber_tag_t request,
	       ber_int_t msgid, void *params);
#ifdef HAVE_LDAP_H
#undef ldap_connect
#define ldap_connect mask_ldap_connect
#endif
LDAP *ldap_connect(LDAPConfig * config);
void free_ldap_user_entry(USEREntry * user);
USEREntry *get_ldap_user_entry(LDAP * ld, LDAPConfig * config, char *uid);
